@@ -1,7 +1,3 @@
-###########################################
-# Archive::Tar::Wrapper -- 2005, Mike Schilli <cpan@perlmeister.com>
-###########################################
-
 ###########################################
 package Archive::Tar::Wrapper;
 ###########################################
@@ -20,7 +16,7 @@ use File::Which qw(which);
 use IPC::Run qw(run);
 use Cwd;
 
-our $VERSION = "0.18";
+our $VERSION = "0.21";
 
 ###########################################
 sub new {
@@ -91,7 +87,7 @@ sub read {
         LOGDIE "Cannot chdir to $self->{tardir}";
 
     my $compr_opt = "";
-    $compr_opt = "z" if $self->is_compressed($tarfile);
+    $compr_opt = $self->is_compressed($tarfile);
 
     my $cmd = [$self->{tar}, "${compr_opt}x$self->{tar_read_options}",
                @{$self->{tar_gnu_read_options}},
@@ -119,18 +115,19 @@ sub is_compressed {
 ###########################################
     my($self, $tarfile) = @_;
 
-    return 1 if $tarfile =~ /\.t?gz$/i;
+    return 'z' if $tarfile =~ /\.t?gz$/i;
+    return 'j' if $tarfile =~ /\.bz2$/i;
 
         # Sloppy check for gzip files
     open FILE, "<$tarfile" or die "Cannot open $tarfile";
     binmode FILE;
     my $read = sysread(FILE, my $two, 2, 0) or die "Cannot sysread";
     close FILE;
-    return 1 if 
+    return 'z' if 
         ord(substr($two, 0, 1)) eq 0x1F and 
         ord(substr($two, 1, 1)) eq 0x8B;
 
-    return 0;
+    return q{};
 }
 
 ###########################################
@@ -221,11 +218,13 @@ sub add {
 ######################################
 sub perm_cp {
 ######################################
+    my($source, $target) = @_;
+
     # Lifted from Ben Okopnik's
     # http://www.linuxgazette.com/issue87/misc/tips/cpmod.pl.txt
 
-    my $perms = perm_get($_[0]);
-    perm_set($_[1], $perms);
+    my $perms = perm_get($source);
+    perm_set($target, $perms);
 }
 
 ######################################
@@ -244,8 +243,10 @@ sub perm_set {
 ######################################
     my($filename, $perms) = @_;
 
-    chown($perms->[1], $perms->[2], $filename) or
-        LOGDIE "Cannot chown $filename ($!)";
+      # ignore errors here, as we can't change uid/gid unless we're
+      # the superuser (see LIMITATIONS section)
+    chown($perms->[1], $perms->[2], $filename);
+
     chmod($perms->[0] & 07777,    $filename) or
         LOGDIE "Cannot chmod $filename ($!)";
 }
@@ -794,6 +795,25 @@ them. This will be fixed, though.
 Filenames containing newlines are causing problems with the list
 iterators. To be fixed.
 
+=item *
+
+If you ask Archive::Tar::Wrapper to add a file to a tarball, it copies it into
+a temporary directory and then calls the system tar to wrap up that directory
+into a tarball.
+
+This approach has limitations when it comes to file permissions: If the file to
+be added belongs to a different user/group, Archive::Tar::Wrapper will adjust
+the uid/gid/permissions of the target file in the temporary directory to
+reflect the original file's settings, to make sure the system tar will add it
+like that to the tarball, just like a regular tar run on the original file
+would. But this will fail of course if the original file's uid is different
+from the current user's, unless the script is running with superuser rights.
+The tar program by itself (without Archive::Tar::Wrapper) works differently:
+It'll just make a note of a file's uid/gid/permissions in the tarball (which it
+can do without superuser rights) and upon extraction, it'll adjust the
+permissions of newly generated files if the -p option is given (default for
+superuser).
+
 =back
 
 =head1 BUGS
